# -*- coding: utf-8 -*-
import pghistory

from django.conf import settings
from django.utils.module_loading import import_string

from thx.pg_history.models import HistoryModelMixin


def track_history(*args, **kwargs):
    """
    pghistory track wrapper. Produce a standardized snapshot label and provides default kwargs.
    If you need customizations, use the pghistory track decorator.
    """
    def _model_wrapper(model_class):
        snaphot = pghistory.Snapshot('snapshot')

        # Monkey patch
        model_class._original_save = model_class.save
        model_class.save = HistoryModelMixin.save
        model_class.prepare_single_fk_context = HistoryModelMixin.prepare_single_fk_context
        model_class.prepare_fk_context = HistoryModelMixin.prepare_fk_context

        kwargs.setdefault("app_label", settings.HISTORY_APP_NAME)
        kwargs.setdefault("exclude", settings.HISTORY_TRACKED_MODELS_EXCLUDED_FIELDS)
        return pghistory.track(snaphot, *args, **kwargs)(model_class)
    return _model_wrapper


def register_track_history_models():
    for model, kwargs in settings.HISTORY_TRACKED_MODELS.items():
        model = import_string(model)
        track_history(**kwargs)(model)
